package com.sucl.sbmp.sys.biz;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * ProjectName: fw-cas
 * Description: jackson 序列化工具
 * User: wangkai <wangkai@feewee.cn>
 * Date: 2019/9/26
 * Time: 2:39 PM
 */
public class MapperUtils {

    private final static ObjectMapper objectMapper = new ObjectMapper();


    public static ObjectMapper getInstance() {
        return objectMapper;
    }

    /**
     * 转换为 JSON 字符串
     *
     * @param obj
     * @return
     * @throws Exception
     */
    public static String obj2json(Object obj) throws Exception {
        return objectMapper.writeValueAsString(obj);
    }

    /**
     * 转换为 JSON 字符串，忽略空值
     *
     * @param obj
     * @return
     * @throws Exception
     */
    public static String obj2jsonIgnoreNull(Object obj) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);
        return mapper.writeValueAsString(obj);
    }

    /**
     * 转换为 JavaBean
     *
     * @param jsonString
     * @param clazz
     * @return
     * @throws Exception
     */
    public static <T> T json2pojo(String jsonString, Class<T> clazz) throws Exception {
        objectMapper.configure(DeserializationFeature.ACCEPT_SINGLE_VALUE_AS_ARRAY, true);
        return objectMapper.readValue(jsonString, clazz);
    }


    /*public static void main(String[] args) throws Exception {
        String content = "{\"metadata\":{},\"dom\":\"fw-erp\",\"cacheMillis\":3000,\"useSpecifiedURL\":false,\"hosts\":[{\"valid\":true,\"marked\":false,\"metadata\":{\"spring.application.config.rest.control.enabled\":\"true\",\"spring.application.group.key\":\"group\",\"spring.application.context-path\":\"/\",\"spring.application.name\":\"fw-erp\",\"spring.application.register.control.enabled\":\"true\",\"spring.application.discovery.version\":\"5.0.8\",\"spring.application.discovery.control.enabled\":\"true\",\"spring.application.discovery.plugin\":\"Nacos\",\"region\":\"prd\",\"preserved.register.source\":\"SPRING_CLOUD\",\"version\":\"1.0\",\"group\":\"discovery-prd-group\"},\"instanceId\":\"172.26.154.156#8010#DEFAULT#DEFAULT_GROUP@@fw-erp\",\"port\":8010,\"healthy\":true,\"ip\":\"172.26.154.156\",\"clusterName\":\"DEFAULT\",\"weight\":1.0,\"ephemeral\":true,\"serviceName\":\"fw-erp\",\"enabled\":true}],\"name\":\"DEFAULT_GROUP@@fw-erp\",\"checksum\":\"e459834d9818261271c66baf60f82f10\",\"lastRefTime\":1569478259464,\"env\":\"\",\"clusters\":\"\"}";
        final Map<String, Object> stringObjectMap = json2mapDeeply(content);
        if (!Objects.isNull(stringObjectMap)) {
            final String string = stringObjectMap.get("hosts").toString();
            System.out.println("hosts======>" + string.length());
        }
        //   System.out.println("serviceInfo======> " + stringObjectMap);


        // 服务挂掉放回json
        String downContent = "{\"metadata\":{},\"dom\":\"fw-erp\",\"cacheMillis\":3000,\"useSpecifiedURL\":false,\"hosts\":[],\"name\":\"DEFAULT_GROUP@@fw-erp\",\"checksum\":\"f9576239b71263214de8c2918cf5d7bb\",\"lastRefTime\":1569481372308,\"env\":\"\",\"clusters\":\"\"}";
        final Map<String, Object> stringObjectMap1 = json2mapDeeply(downContent);
        if (!Objects.isNull(stringObjectMap1)) {
            final String string = stringObjectMap1.get("hosts").toString();
            System.out.println("hosts======>" + string.length());
        }
        //   System.out.println("serviceInfo======> " + stringObjectMap1);

        final HttpClientResult httpClientResult = HttpClientUtils.doGet(MessageFormat.format("http://192.168.0.230:8848/nacos/v1/ns/instance/list?serviceName={0}&namespaceId=df959b8c-de58-4d02-b9fb-d65ca3be05f3", "fw-erp"));
        final String content1 = httpClientResult.getContent();
        final Map<String, Object> stringObjectMap2 = json2mapDeeply(content1);
        if (!Objects.isNull(stringObjectMap2)) {
            final String string = stringObjectMap2.get("hosts").toString();
            System.out.println("hosts======>" + string.length());
        }

        final String replace = StringUtils.replace("lb://fw-activity", "lb://", "");

        System.out.println(replace);
    }*/


    /**
     * 字符串转换为 Map<String, Object>
     *
     * @param jsonString
     * @return
     * @throws Exception
     */
    public static <T> Map<String, Object> json2map(String jsonString) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);
        return mapper.readValue(jsonString, Map.class);
    }

    /**
     * 字符串转换为 Map<String, T>
     */
    public static <T> Map<String, T> json2map(String jsonString, Class<T> clazz) throws Exception {
        Map<String, Map<String, Object>> map = objectMapper.readValue(jsonString, new TypeReference<Map<String, T>>() {
        });
        Map<String, T> result = new HashMap<String, T>();
        for (Map.Entry<String, Map<String, Object>> entry : map.entrySet()) {
            result.put(entry.getKey(), map2pojo(entry.getValue(), clazz));
        }
        return result;
    }

    /**
     * 深度转换 JSON 成 Map
     *
     * @param json
     * @return
     */
    public static Map<String, Object> json2mapDeeply(String json) throws Exception {
        return json2MapRecursion(json, objectMapper);
    }

    /**
     * 把 JSON 解析成 List，如果 List 内部的元素存在 jsonString，继续解析
     *
     * @param json
     * @param mapper 解析工具
     * @return
     * @throws Exception
     */
    private static List<Object> json2ListRecursion(String json, ObjectMapper mapper) throws Exception {
        if (json == null) {
            return null;
        }

        List<Object> list = mapper.readValue(json, List.class);

        for (Object obj : list) {
            if (obj != null && obj instanceof String) {
                String str = (String) obj;
                if (str.startsWith("[")) {
                    obj = json2ListRecursion(str, mapper);
                } else if (obj.toString().startsWith("{")) {
                    obj = json2MapRecursion(str, mapper);
                }
            }
        }

        return list;
    }

    /**
     * 把 JSON 解析成 Map，如果 Map 内部的 Value 存在 jsonString，继续解析
     *
     * @param json
     * @param mapper
     * @return
     * @throws Exception
     */
    private static Map<String, Object> json2MapRecursion(String json, ObjectMapper mapper) throws Exception {
        if (json == null) {
            return null;
        }

        Map<String, Object> map = mapper.readValue(json, Map.class);

        for (Map.Entry<String, Object> entry : map.entrySet()) {
            Object obj = entry.getValue();
            if (obj != null && obj instanceof String) {
                String str = ((String) obj);

                if (str.startsWith("[")) {
                    List<?> list = json2ListRecursion(str, mapper);
                    map.put(entry.getKey(), list);
                } else if (str.startsWith("{")) {
                    Map<String, Object> mapRecursion = json2MapRecursion(str, mapper);
                    map.put(entry.getKey(), mapRecursion);
                }
            }
        }

        return map;
    }

    /**
     * 将 JSON 数组转换为集合
     *
     * @param jsonArrayStr
     * @param clazz
     * @return
     * @throws Exception
     */
    public static <T> List<T> json2list(String jsonArrayStr, Class<T> clazz) throws Exception {
        JavaType javaType = getCollectionType(ArrayList.class, clazz);
        List<T> list = (List<T>) objectMapper.readValue(jsonArrayStr, javaType);
        return list;
    }


    /**
     * 获取泛型的 Collection Type
     *
     * @param collectionClass 泛型的Collection
     * @param elementClasses  元素类
     * @return JavaType Java类型
     * @since 1.0
     */
    public static JavaType getCollectionType(Class<?> collectionClass, Class<?>... elementClasses) {
        return objectMapper.getTypeFactory().constructParametricType(collectionClass, elementClasses);
    }

    /**
     * 将 Map 转换为 JavaBean
     *
     * @param map
     * @param clazz
     * @return
     */
    public static <T> T map2pojo(Map map, Class<T> clazz) {
        return objectMapper.convertValue(map, clazz);
    }

    /**
     * 将 Map 转换为 JSON
     *
     * @param map
     * @return
     */
    public static String mapToJson(Map map) {
        try {
            return objectMapper.writeValueAsString(map);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

    /**
     * 将 JSON 对象转换为 JavaBean
     *
     * @param obj
     * @param clazz
     * @return
     */
    public static <T> T obj2pojo(Object obj, Class<T> clazz) {
        return objectMapper.convertValue(obj, clazz);
    }
}
