package com.sucl.sbmp.sys.biz;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.google.common.collect.Maps;
import com.sucl.sbmp.sys.entity.OldSeries;
import com.sucl.sbmp.sys.entity.OldSpec;
import com.sucl.sbmp.sys.entity.TempCarSeries;
import com.sucl.sbmp.sys.service.OldSeriesService;
import com.sucl.sbmp.sys.service.OldSpecService;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.MessageFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 老版EPC备件等数据抓取
 * <p>
 * create at 2019-12-31
 *
 * @author 钟奉池
 */
@Service
public class OldChanganAutoEpcPartsBiz {
    /**
     * logger
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(ChanganAutoEpcPartsBiz.class);
    /**
     * Authorization
     */
    private final static String Authorization = "243e20495bf74fff89028ad8b03e6bb1";
    private final static Long CHANG_AN_BRAND_ID = 1L;  //长安汽车
    private final static Long OU_SHANG_BRAND_ID = 19L;  //欧尚汽车
    /**
     * EPC车系服务
     */
    private final OldSeriesService oldSeriesService;
    /**
     * EPC车型服务
     */
    private final OldSpecService oldSpecService;
    /**
     * 默认构造器
     */
    @Autowired
    public OldChanganAutoEpcPartsBiz(final OldSeriesService oldSeriesService,
                                     final OldSpecService oldSpecService){
        this.oldSeriesService = oldSeriesService;
        this.oldSpecService = oldSpecService;
    }
    /**
     * 设置Token
     */
    public HashMap<String, String> setToken(final String token){
        // token
        final HashMap<String, String> requestHeader = Maps.newHashMap();
        requestHeader.put("Authorization", token);
        return requestHeader;
    }
    /**
     * 获取车系列表
     */
    public void selectSeriesByBrand(final HashMap<String, String> requestHeader){
        String path = "http://epc.changan.com.cn:8080//oled/epc-tb-categories/selectModel?brandId=1&_" + System.currentTimeMillis();
        try{
            final HttpClientResult httpClientResult = HttpClientUtils.doGet(path, requestHeader);
            final String content = httpClientResult.getContent();
            if (StringUtils.isNotEmpty(content)) {
                final JSONObject jsonObject = JSON.parseObject(content);
                final JSONArray data = jsonObject.getJSONArray("data");
                final List<OldSeries> seriesList = JSONObject.parseArray(data.toJSONString(), OldSeries.class);
                LOGGER.info("长安汽车或长安欧尚车系数据：【{}】", seriesList);
                if (CollectionUtils.isEmpty(seriesList)) {
                    return;
                }
                seriesList.forEach(oldSeries -> {
                    oldSeries.setBrandId(CHANG_AN_BRAND_ID);
                    oldSeries.setCreateTime(new Date());
                });
                // 保存数据
                this.oldSeriesService.saveBatch(seriesList);
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }
    /**
     * 根据车系ID获取车型编码
     */
    public void selectScCodeBySeriesId(final HashMap<String, String> requestHeader){
        String path2 = "http://epc.changan.com.cn:8080/oled/epc-tb-categories/selectSc?modelId={0}&_" + System.currentTimeMillis();
        // 获取所有的车系，遍历得到seriesId,没循环一次请求一下，中间间隔2秒继续下次请求
        final List<OldSeries> carSeries = this.oldSeriesService.list();
        if (CollectionUtils.isEmpty(carSeries)) {
            return;
        }
        for (OldSeries series : carSeries) {
            final String url = MessageFormat.format(path2, series.getSeriesId());
            try {
                final HttpClientResult httpClientResult = HttpClientUtils.doGetWithHeader(url, requestHeader);
                //
                final String content = httpClientResult.getContent();
                if (StringUtils.isNotEmpty(content)) {
                    final JSONObject jsonObject = JSON.parseObject(content);
                    final JSONArray data = jsonObject.getJSONArray("data");
                    final List<String> codes = JSONObject.parseArray(data.toJSONString(), String.class);
                    if (CollectionUtils.isNotEmpty(codes)) {
                        final List<OldSpec> oldSpecList = codes.stream().map(code -> {
                            OldSpec tempScCode = new OldSpec();
                            tempScCode.setCreateTime(new Date());
                            tempScCode.setSpecCode(code);
                            tempScCode.setSeriesId(series.getSeriesId());
                            return tempScCode;
                        }).collect(Collectors.toList());
                        // 保存单个车系的车型代码列表
                        this.oldSpecService.saveBatch(oldSpecList);
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            try {
                java.lang.Thread.sleep(2);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
