package com.sucl.sbmp.sys.biz;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.function.BinaryOperator;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 公共帮助服务
 * <p>
 * create at 2020-05-27
 *
 * @author baiyun
 */
public class U {
    /**
     * string value ','
     */
    public static final String COMMA = ",";
    public static final String EMPTY = "";
    /**
     * 将集合制定字段转Map键值对
     *
     * @param coll    集合对象
     * @param column1 列1
     * @param column2 列2
     * @return 返回键值对
     */
    public static <P, R1, R2> Map<R1, List<R2>> transfersMap(Collection<P> coll, Function<P, R1> column1, Function<P, R2> column2) {
        if (CollectionUtils.isEmpty(coll) || Objects.isNull(column1) || Objects.isNull(column2)) {
            return new HashMap<>();
        }
        return coll.stream().collect(Collectors.toMap(column1,
                i -> new ArrayList<>(Collections.singletonList(column2.apply(i))), (i1, i2) -> {
                    i1.addAll(i2);
                    return i1;
                }));
    }

    /**
     * 将集合制定字段转Map键值对
     *
     * @param coll   集合对象
     * @param column 列
     * @return 返回键值对
     */
    public static <P, R> Map<R, List<P>> transfersMap(Collection<P> coll, Function<P, R> column) {
        if (CollectionUtils.isEmpty(coll) || Objects.isNull(column)) {
            return new HashMap<>();
        }
        return coll.stream().filter(it -> Objects.nonNull(column.apply(it))).collect(Collectors.groupingBy(column));
    }

    /**
     * 将集合制定字段转Map键值对
     *
     * @param coll   集合对象
     * @param column 列
     * @return 返回键值对
     */
    public static <P, R> Map<R, P> transferMap(Collection<P> coll, Function<P, R> column) {
        if (CollectionUtils.isEmpty(coll) || Objects.isNull(column)) {
            return new HashMap<>();
        }
        return coll.stream().collect(Collectors.toMap(column, i -> i, (i1, i2) -> i1));
    }

    /**
     * 将集合制定字段转Map键值对
     *
     * @param coll    集合对象
     * @param column1 列
     * @param column2 列
     * @return 返回键值对
     */
    public static <P, R> Map<R, Integer> transferSumMap(Collection<P> coll, Function<P, R> column1, Function<P, Integer> column2) {
        if (CollectionUtils.isEmpty(coll) || Objects.isNull(column1) || Objects.isNull(column2)) {
            return new HashMap<>();
        }
        return coll.stream().collect(Collectors.toMap(column1, column2, (i1, i2) -> i1 + i1));
    }

    /**
     * 将集合制定字段转Map键值对
     *
     * @param coll    集合对象
     * @param column1 列1
     * @param column2 列2
     * @return 返回键值对
     */
    public static <P, R1, R2, R3> Map<R1, Map<R2, R3>> transferMap(Collection<P> coll, Function<P, R1> column1, Function<P, R2> column2, Function<P, R3> column3) {
        if (CollectionUtils.isEmpty(coll) || Objects.isNull(column1) || Objects.isNull(column2) || Objects.isNull(column3)) {
            return new HashMap<>();
        }
        final Map<R1, Map<R2, R3>> map = new HashMap<>();
        coll.stream().collect(Collectors.groupingBy(column1)).forEach((k, v) -> map.put(k, transferMap(v, column2, column3)));
        return map;
    }

    /**
     * 将集合制定字段转Map键值对
     *
     * @param coll    集合对象
     * @param column1 列1
     * @param column2 列2
     * @return 返回键值对
     */
    public static <P, R1, R2> Map<R1, Map<R2, P>> transferToMap(Collection<P> coll, Function<P, R1> column1, Function<P, R2> column2) {
        if (CollectionUtils.isEmpty(coll) || Objects.isNull(column1) || Objects.isNull(column2)) {
            return new HashMap<>();
        }
        final Map<R1, Map<R2, P>> map = new HashMap<>();
        coll.stream().collect(Collectors.groupingBy(column1)).forEach((k, v) -> map.put(k,
                v.stream().collect(Collectors.toMap(column2, i -> i, (i1, i2) -> i1))));
        return map;
    }

    /**
     * 将集合制定字段转Map键值对
     *
     * @param coll    集合对象
     * @param column1 列1
     * @param column2 列2
     * @return 返回键值对
     */
    public static <P, R, R1, R2, R3> Map<R1, Map<R2, R3>> transferToMap(Collection<P> coll, Function<P, List<R>> column, Function<P, R1> column1, Function<R, R2> column2, Function<R, R3> column3) {
        if (CollectionUtils.isEmpty(coll) || Objects.isNull(column) || Objects.isNull(column1) || Objects.isNull(column2) || Objects.isNull(column3)) {
            return new HashMap<>();
        }
        final Map<R1, Map<R2, R3>> map = new HashMap<>();
        coll.stream().collect(Collectors.groupingBy(column1)).forEach((k, v) -> map.put(k,
                v.stream().map(column).flatMap(Collection::parallelStream)
                        .collect(Collectors.toMap(column2, column3, (i1, i2) -> i1))));
        return map;
    }

    /**
     * 将集合制定字段转Map键值对
     *
     * @param coll    集合对象
     * @param column1 列1
     * @param column2 列2
     * @return 返回键值对
     */
    public static <P, R1, R2> Map<R1, R2> transferMap(Collection<P> coll, Function<P, R1> column1, Function<P, R2> column2) {
        if (CollectionUtils.isEmpty(coll) || Objects.isNull(column1) || Objects.isNull(column2)) {
            return new HashMap<>();
        }
        return coll.stream().filter(Objects::nonNull).collect(Collectors.toMap(column1, column2, (i1, i2) -> i1));
    }

    /**
     * 将集合制定字段转Map键值对
     *
     * @param coll    集合对象
     * @param column1 列1
     * @param column2 列2
     * @return 返回键值对
     */
    public static <P, R1, R2> Map<R1, R2> transferMap(Collection<P> coll, Function<P, R1> column1, Function<P, R2> column2, BinaryOperator<R2> mergeFunction) {
        if (CollectionUtils.isEmpty(coll) || Objects.isNull(column1) || Objects.isNull(column2)) {
            return new HashMap<>();
        }
        return coll.stream().collect(Collectors.toMap(column1, column2, mergeFunction));
    }

    /**
     * 将ID字符串转换为id集合
     *
     * @param ids ID集合字符串","分割
     * @return id集合
     */
    public static List<Long> strToIds(String ids) {
        if (StringUtils.isEmpty(ids)) {
            return new ArrayList<>();
        }
        return Arrays.stream(ids.split(COMMA))
                .filter(it -> !StringUtils.isEmpty(it))
                .map(Long::valueOf)
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * 将ID字符串转换为id集合
     *
     * @param ids ID集合字符串","分割
     * @return id集合
     */
    public static List<String> strToStrs(String ids) {
        if (StringUtils.isEmpty(ids)) {
            return new ArrayList<>();
        }
        return Arrays.stream(ids.split(COMMA))
                .filter(it -> !StringUtils.isEmpty(it))
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * 将ID字符串转换为id集合
     *
     * @param coll 对象集合
     * @return String集合
     */
    public static <P> Set<String> strsToStrs(Collection<P> coll, Function<P, String> column) {
        if (CollectionUtils.isEmpty(coll)) {
            return new HashSet<>();
        }
        return coll.stream().map(column)
                .filter(it -> !StringUtils.isEmpty(it))
                .distinct()
                .map(U::strToStrs)
                .flatMap(Collection::stream)
                .collect(Collectors.toSet());
    }

    /**
     * 将列表固定字段ID字符串转换为id集合
     *
     * @param coll 对象集合
     * @return id集合
     */
    public static <P> List<Long> strToIds(Collection<P> coll, Function<P, String> column) {
        if (CollectionUtils.isEmpty(coll)) {
            return new ArrayList<>();
        }
        return coll.stream().map(column)
                .filter(it -> !StringUtils.isEmpty(it))
                .map(U::strToIds)
                .flatMap(Collection::stream)
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * 将列表固定字段ID集合转换为id字符串集合
     *
     * @param coll 对象集合
     * @return id字符串集合
     */
    public static <P> List<String> idsToStrs(Collection<P> coll, Function<P, Long> column) {
        if (CollectionUtils.isEmpty(coll)) {
            return new ArrayList<>();
        }
        return coll.stream().map(column)
                .filter(Objects::nonNull)
                .map(String::valueOf)
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * 将ID集合转换为id字符串集合
     *
     * @param ids ID集合
     * @return id 字符串集合
     */
    public static List<String> idsToStrs(List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return new ArrayList<>();
        }
        return ids.stream()
                .filter(Objects::nonNull)
                .map(String::valueOf)
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * 将二维数组转换为一维数组
     *
     * @param list ID集合
     * @return 一维数组
     */
    public static <T> List<T> flatMap(Collection<List<T>> list) {
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream()
                .flatMap(Collection::stream)
                .filter(Objects::nonNull)
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * 获取集合制定字段集合
     *
     * @param coll 集合
     * @return 字段集合
     */
    public static <P, R> List<R> collIds(Collection<P> coll, Function<P, R> column) {
        if (CollectionUtils.isEmpty(coll)) {
            return new ArrayList<>();
        }
        return coll.stream()
                .map(column)
                .filter(Objects::nonNull)
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * 根据指定字段分组
     *
     * @param coll 集合
     * @return 字段集合
     */
    public static <P, R> Map<R, List<P>> groupBy(Collection<P> coll, Function<P, R> column) {
        if (CollectionUtils.isEmpty(coll) || Objects.isNull(column)) {
            return new HashMap<>();
        }
        return coll.stream()
                .collect(Collectors.groupingBy(column));
    }

    /**
     * 根据条件过滤集合数据
     *
     * @param coll 集合
     * @return 过滤后集合
     */
    public static <P> List<P> filter(Collection<P> coll, Predicate<P> column) {
        if (CollectionUtils.isEmpty(coll)) {
            return new ArrayList<>();
        }
        return coll.stream()
                .filter(column)
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * 获取集合制定字段集合
     *
     * @param coll 集合
     * @return 字段集合
     */
    public static <P, R> Set<R> colls(Collection<P> coll, Function<P, Collection<R>> column) {
        if (CollectionUtils.isEmpty(coll)) {
            return new HashSet<>();
        }
        return coll.stream()
                .map(column)
                .flatMap(Collection::parallelStream)
                .collect(Collectors.toSet());
    }

    /**
     * 获取集合制定字段集合
     *
     * @param coll 集合
     * @return 字段集合
     */
    public static <P, R> Set<R> toSets(Collection<P> coll, Function<P, R> column) {
        if (CollectionUtils.isEmpty(coll)) {
            return new HashSet<>();
        }
        return coll.stream()
                .map(column)
                .collect(Collectors.toSet());
    }

    /**
     * 获取集合制定字段集合
     *
     * @param coll 集合
     * @return 字段集合
     */
    public static <P, R1, R2> List<R2> collIds(Collection<P> coll, Function<P, List<R1>> column1, Function<R1, R2> column2) {
        if (CollectionUtils.isEmpty(coll)) {
            return new ArrayList<>();
        }
        return coll.stream()
                .map(column1)
                .flatMap(Collection::parallelStream)
                .map(column2)
                .collect(Collectors.toList());
    }

    /**
     * 根据多个字段去重
     */
    @SafeVarargs
    public static <P, R> List<P> distinct(Collection<P> coll, Function<P, R>... columns) {
        if (CollectionUtils.isEmpty(coll)) {
            return new ArrayList<>();
        }
        Map<String, P> map = transferMap(coll, it -> {
            StringBuilder sb = new StringBuilder();
            for (Function<P, R> column : columns) {
                sb.append(String.format("%s_", column.apply(it)));
            }
            return sb.toString();
        });
        return new ArrayList<>(map.values());
    }

    /**
     * 将将ID集合转字符串
     *
     * @param coll 集合
     * @return 字段集合
     */
    public static String idsToStr(Collection<Long> coll) {
        if (CollectionUtils.isEmpty(coll)) {
            return EMPTY;
        }
        return coll.stream()
                .map(String::valueOf)
                .distinct()
                .collect(Collectors.joining(COMMA));
    }

    /**
     * 将String集合转字符串
     *
     * @param coll 集合
     * @return 字段集合
     */
    public static String strsToStr(Collection<String> coll) {
        if (CollectionUtils.isEmpty(coll)) {
            return EMPTY;
        }
        return coll.stream()
                .map(String::valueOf)
                .distinct()
                .collect(Collectors.joining(COMMA));
    }

    /**
     * 将将ID集合转字符串
     *
     * @param coll 集合
     * @return 字段集合
     */
    public static String collToStr(Collection<String> coll) {
        if (CollectionUtils.isEmpty(coll)) {
            return EMPTY;
        }
        return coll.stream()
                .map(String::valueOf)
                .distinct()
                .collect(Collectors.joining(COMMA));
    }

    /**
     * 将将ID集合转字符串
     *
     * @param coll 集合
     * @return 字段集合
     */
    public static String collIdToStr(Collection<Long> coll) {
        if (CollectionUtils.isEmpty(coll)) {
            return EMPTY;
        }
        return coll.stream()
                .map(String::valueOf)
                .distinct()
                .collect(Collectors.joining(COMMA));
    }

    /**
     * 列表指定字段求和
     *
     * @param coll    集合
     * @param column1 指定列
     * @return 求和后列表
     */
    public static <P, R1> List<P> collsColumnSum(Collection<P> coll, Function<P, R1> column1, Function<List<P>, P> column2) {
        if (CollectionUtils.isEmpty(coll) || Objects.isNull(column1)) {
            return new ArrayList<>();
        }
        return coll.stream()
                .collect(Collectors.groupingBy(column1))
                .values()
                .stream().map(column2)
                .collect(Collectors.toList());
    }

    /**
     * 列表汇总
     *
     * @param lists 可变参数列表
     * @return 汇总后的列表
     */
    @SafeVarargs
    public static <P> List<P> collSummary(List<P>... lists) {
        Stream<P> stream = Stream.empty();
        for (List<P> values : lists) {
            stream = Stream.concat(stream, values.stream());
        }
        return stream.collect(Collectors.toList());
    }

    /**
     * 从列表中筛选出指定字段，并可以设置默认值
     */
    public static <P, R> R find(List<P> lists, Function<P, R> column, Predicate<P> filter, R defaultValue) {
        if (CollectionUtils.isEmpty(lists) || Objects.isNull(filter)) {
            return defaultValue;
        }
        return lists.stream().filter(filter).map(column).findFirst().orElse(defaultValue);
    }

    /**
     * 分组数据
     *
     * @param data 数据列表
     * @return 分组后的数据列表
     */
    public static <P> List<List<P>> groupData(final List<P> data, final int cnt) {
        int num = Math.max(cnt, 10);
        final List<List<P>> list = new ArrayList<>();
        List<P> bscs = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            bscs.add(data.get(i));
            if (i % num == 0) {
                list.add(bscs);
                bscs = new ArrayList<>();
            }
        }
        if (bscs.size() > 0) {
            list.add(bscs);
        }
        return list;
    }

    /**
     * 分组更新数据
     *
     * @param data 数据列表
     */
    public static <P> void groupUpdateData(final List<P> data, Consumer<List<P>> consumer) {
        if (CollectionUtils.isEmpty(data)) {
            return;
        }
        final List<List<P>> list = groupData(data, 100);
        list.forEach(consumer);
    }

    /**
     * 数据排序
     *
     * @param coll 集合
     */
    public static <P, R> List<P> sorted(Collection<P> coll, Function<P, R> column, Comparator<R> dateComparator, Boolean isAsc) {
        if (CollectionUtils.isEmpty(coll)) {
            return new ArrayList<>();
        }
        List<P> list = new ArrayList<>();
        list.addAll(coll.stream()
                .filter(it -> Objects.nonNull(column.apply(it)))
                .sorted(isAsc ? Comparator.comparing(column, dateComparator) :
                        Comparator.comparing(column, dateComparator).reversed())
                .collect(Collectors.toList()));
        list.addAll(filter(coll, it -> Objects.isNull(column.apply(it))));
        return list;
    }

    public static Long strOne(String str) {
        List<Long> ids = strToIds(str);
        if (CollectionUtils.isEmpty(ids)) {
            return null;
        }
        return ids.get(0);
    }

    // 计算两个集合的交集
    public static <P> Set<P> intersection(Collection<P> coll1, Collection<P> coll2) {
        Set<P> res = new HashSet<>();
        if (CollectionUtils.isEmpty(coll1) || CollectionUtils.isEmpty(coll2)) {
            return res;
        }
        coll1.forEach(it -> {
            if (coll2.contains(it)) {
                res.add(it);
            }
        });
        return res;
    }

    /**
     * 分页
     *
     * @param coll 集合
     * @return 过滤后集合
     */
    public static <P> List<P> page(Collection<P> coll, int current, int pageSize) {
        if (CollectionUtils.isEmpty(coll)) {
            return new ArrayList<>();
        }
        long pa = (long) (current - 1) * pageSize;
        return coll.stream()
                .skip(Math.max(pa, 0))
                .limit(pageSize)
                .collect(Collectors.toList());
    }
}